/*********************************************************************
*                    SEGGER Microcontroller GmbH                     *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*            (c) 2014 - 2018 SEGGER Microcontroller GmbH             *
*                                                                    *
*           www.segger.com     Support: support@segger.com           *
*                                                                    *
**********************************************************************
*                                                                    *
* All rights reserved.                                               *
*                                                                    *
* Redistribution and use in source and binary forms, with or         *
* without modification, are permitted provided that the following    *
* conditions are met:                                                *
*                                                                    *
* - Redistributions of source code must retain the above copyright   *
*   notice, this list of conditions and the following disclaimer.    *
*                                                                    *
* - Neither the name of SEGGER Microcontroller GmbH                  *
*   nor the names of its contributors may be used to endorse or      *
*   promote products derived from this software without specific     *
*   prior written permission.                                        *
*                                                                    *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND             *
* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,        *
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF           *
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE           *
* DISCLAIMED.                                                        *
* IN NO EVENT SHALL SEGGER Microcontroller GmbH BE LIABLE FOR        *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR           *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  *
* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;    *
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF      *
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT          *
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE  *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH   *
* DAMAGE.                                                            *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------

File    : main.c
Purpose : Generic application start

*/

#include <stdio.h>
#include <stdlib.h>

/*********************************************************************
*
*       main()
7*
*  Function description
*   Application entry point.
*/
extern const unsigned char hello_110118_array[];
unsigned char *fixed_ptr = 0x00100000;
unsigned long os_dicnt;
unsigned long basepri = 0x0;
volatile unsigned long *ul_ptr;
volatile void *v_ptr;

  #define os_di() { __asm volatile ("movs r0, #128    \n\t"  \
                                    "msr  BASEPRI, r0 \n\t"  \
                                    "nop              \n\t"  \
                                    "nop              \n\t"  \
                                    :                        \
                                    :                        \
                                    : "r0"                   \
                                    );                       \
                  }

void disable_irq(void)
{
   __asm volatile ("cpsid i":::"memory");
}
void os_assert(void)
{
   if  (!(os_dicnt & 0xf0))
   { 
      printf ("os_dicnt %x\n", os_dicnt);
   }
}

void os_assert_dicnt(void)
{
  os_assert();
}

void os_incdi(void)
{
  os_assert_dicnt();
  os_di();
  disable_irq();
  os_dicnt++;
}

void __set_MSP(unsigned long topOfMainStack)
{
  __asm volatile ("MSR msp, %0\n"::"r"(topOfMainStack):"sp");
}

void __set_CONTROL(unsigned long control)
{
   asm ("MSR control, %0"::"r" (control):"memory");
}


#define THUMB_BIT 1
#define BOOT_START_ADDR 0x00100000
#define BOOT_STACK_PTR  (*(volatile unsigned *) (BOOT_START_ADDR + 0x00))
#define BOOT_RESET_PTR  (*(volatile unsigned *) (BOOT_START_ADDR + 0x04))
#define BOOT_RESET_THUMB_PTR (*(volatile unsigned *) (BOOT_START_ADDR + 0x04) |THUMB_BIT)
void (*AppPtr)(void);

#define SCS_BASE (0xE000E000)
#define SCB_BASE (SCS_BASE + 0x0d00)
#define SCB_VTOR (SCB_BASE + 0x0008) // CPUID, ICSR, VTOR, ...


void main(void) {
  int i;

  for (i = 0; i < 100; i++) {
    printf("Hello World %d!\n", i);
  }

  for (i = 0; i< 109840 ;i++)
{
   *fixed_ptr = hello_110118_array[i];
   printf("%x %x %x\n", fixed_ptr, *fixed_ptr, hello_110118_array[i]);

   fixed_ptr++;
}

  
   os_incdi();
   AppPtr  = (void(*)(void)) BOOT_RESET_THUMB_PTR; //(MCP_BOOT_RESET_PTR | THUMB_BIT);
   
   ul_ptr = SCB_VTOR;
   *ul_ptr = BOOT_START_ADDR;
   printf("%x\n", ul_ptr);
   printf("%x\n", *ul_ptr);

   __set_MSP(BOOT_STACK_PTR);
   __set_CONTROL(0x00);
   AppPtr();
  
  
  
  do {
    i++;
  } while (1);
}

/*************************** End of file ****************************/
